package de.devloop.mavor;

import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;
import java.util.Properties;

public class Configuration {

    private static final String DEVELOPMENT_PROPERTIES = "development.properties";

    private static final String ENV_MAVEN_EXECUTABLE = "MAVOR_MAVEN_EXECUTABLE";
    private static final String ENV_TEMP_DIR = "MAVOR_TEMP_DIR";

    private static final String ENV_OPENID_CLIENT_ID = "MAVOR_OPENID_CLIENT_ID";
    private static final String ENV_OPENID_CLIENT_SECRET = "MAVOR_OPENID_CLIENT_SECRET";
    private static final String ENV_OPENID_REDIRECT_URL = "MAVOR_OPENID_REDIRECT_URL";
    private static final String ENV_OPENID_AUTH_URL = "MAVOR_OPENID_AUTH_URL";
    private static final String ENV_OPENID_TOKEN_URL = "MAVOR_OPENID_TOKEN_URL";
    private static final String ENV_OPENID_USERINFO_URL = "MAVOR_OPENID_USERINFO_URL";

    private String mavenExecutable;
    private String tempDir;
    private String openIdClientId;
    private String openIdClientSecret;
    private String openIdRedirectUrl;
    private String openIdAuthUrl;
    private String openIdTokenUrl;
    private String openIdUserInfoUrl;

    public Configuration() throws IOException {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        InputStream developmentPropertiesStream = classLoader.getResourceAsStream(DEVELOPMENT_PROPERTIES);
        if (Objects.nonNull(developmentPropertiesStream)) {
            Properties developmentProperties = new Properties();
            developmentProperties.load(developmentPropertiesStream);
            initByProperties(developmentProperties);
        } else {
            initByProperties(System.getProperties());
        }
    }

    private void initByProperties(Properties properties) {
        mavenExecutable = getNullSafeProperty(properties, ENV_MAVEN_EXECUTABLE);
        tempDir = getNullSafeProperty(properties, ENV_TEMP_DIR);
        openIdClientId = getNullSafeProperty(properties, ENV_OPENID_CLIENT_ID);
        openIdClientSecret = getNullSafeProperty(properties, ENV_OPENID_CLIENT_SECRET);
        openIdRedirectUrl = getNullSafeProperty((properties), ENV_OPENID_REDIRECT_URL);
        openIdAuthUrl = getNullSafeProperty(properties, ENV_OPENID_AUTH_URL);
        openIdTokenUrl = getNullSafeProperty(properties, ENV_OPENID_TOKEN_URL);
        openIdUserInfoUrl = getNullSafeProperty(properties, ENV_OPENID_USERINFO_URL);
    }

    private String getNullSafeProperty(Properties properties, String key) {
        return Objects.requireNonNull(properties.getProperty(key));
    }

    public String getMavenExecutable() {
        return mavenExecutable;
    }

    public String getTempDir() {
        return tempDir;
    }

    public String getOpenIdClientId() {
        return openIdClientId;
    }

    public String getOpenIdClientSecret() {
        return openIdClientSecret;
    }
    
    public String getOpenIdRedirectUrl() {
        return openIdRedirectUrl;
    }

    public String getOpenIdAuthUrl() {
        return openIdAuthUrl;
    }

    public String getOpenIdTokenUrl() {
        return openIdTokenUrl;
    }

    public String getOpenIdUserInfoUrl() {
        return openIdUserInfoUrl;
    }
}