package de.devloop.mavor;

import jakarta.servlet.http.HttpSession;

public class Session {
    private HttpSession session;

    private static final String ATTRIBUTE_USERNAME = "username";
    private static final String ATTRIBUTE_OAUTH_STATE = "oauth.state";
    private static final String ATTRIBUTE_OAUTH_TOKEN = "oauth.token";

    public Session(HttpSession session) {
        this.session = session;
    }

    private String getSafeString(String parameter) {
        Object value = session.getAttribute(parameter);

        if (value != null) {
            return value.toString();
        } else {
            return null;
        }
    }

    public Boolean isAuthenticated() {
        return getUsername() != null;
    }

    public String getUsername() {
        return getSafeString(ATTRIBUTE_USERNAME);
    }

    public void setUsername(String username) {
        session.setAttribute(ATTRIBUTE_USERNAME, username);
    }

    public void setOAuthState(String state) {
        session.setAttribute(ATTRIBUTE_OAUTH_STATE, state);
    }

    public String getOAuthState() {
        return getSafeString(ATTRIBUTE_OAUTH_STATE);
    }

    public void clearOAuthState() {
        session.removeAttribute(ATTRIBUTE_OAUTH_STATE);
    }

    public void setOAuthToken(String token) {
        session.setAttribute(ATTRIBUTE_OAUTH_TOKEN, token);
    }

    public String getOAuthToken() {
        return getSafeString(ATTRIBUTE_OAUTH_TOKEN);
    }
}
