package de.devloop.mavor.servlet;

import java.io.IOException;

import de.devloop.mavor.Configuration;
import de.devloop.mavor.Session;
import de.devloop.openid.AuthenticationUrl;
import de.devloop.openid.OpenID;
import de.devloop.openid.OpenIdConfiguration;
import de.devloop.openid.OpenIdRequestException;
import de.devloop.openid.Token;
import de.devloop.openid.UserInfo;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

@WebServlet("/authenticate")
public class Authentication extends HttpServlet {

    private static final String PARAMETER_STATE = "state";
    private static final String PARAMETER_CODE = "code";

    private OpenIdConfiguration openIdConfiguration;
    private OpenID openID;

    @Override
    public void init() throws ServletException {
        Configuration configuration;
        try {
            configuration = new Configuration();
        } catch (IOException e) {
            throw new ServletException("Configuration Error", e);
        }

        openIdConfiguration = new OpenIdConfiguration();
        openIdConfiguration.setAuthUrl(configuration.getOpenIdAuthUrl());
        openIdConfiguration.setClientId(configuration.getOpenIdClientId());
        openIdConfiguration.setClientSecret(configuration.getOpenIdClientSecret());
        openIdConfiguration.setRedirectUrl(configuration.getOpenIdRedirectUrl());
        openIdConfiguration.setTokenUrl(configuration.getOpenIdTokenUrl());
        openIdConfiguration.setUserInfoUrl(configuration.getOpenIdUserInfoUrl());

        openID = new OpenID(openIdConfiguration);
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        Session session = new Session(req.getSession(true));
        if (req.getParameter(PARAMETER_STATE) != null && req.getParameter(PARAMETER_CODE) != null) {
            if (req.getParameter(PARAMETER_STATE).equals(session.getOAuthState())) {
                session.clearOAuthState();

                Token token;
                UserInfo userInfo;
                try {
                    token = openID.requestToken(req.getParameter(PARAMETER_CODE));
                    userInfo = openID.requestUserInfo(token);
                } catch (OpenIdRequestException e) {
                    throw new ServletException("Login failed", e);
                }
                
                session.setOAuthToken(token.getAccessToken());
                session.setUsername(userInfo.getEmail());
                resp.sendRedirect("/mavor/");
            } else {
                throw new ServletException("OpenID state mismatch!");
            }
        } else {
            AuthenticationUrl authenticationUrl = openID.getAuthenticationUrl();
            session.setOAuthState(authenticationUrl.getState());
            resp.sendRedirect(authenticationUrl.getUrl());
        }
    }
}
